@@ -4,17 +4,72 @@
 import datetime 
 import os
 import sys
+from sqlitedave_package.sqlitedave import sqlite_db
 from postgresdave_package.postgresdave import postgres_db 
 from mysqldave_package.mysqldave import mysql_db 
 from garbledave_package.garbledave import garbledave 
 
+def main():
+	obj = schemawiz()
+	#obj.loadcsvfile('sample7.csv')
+	#print(obj.guess_mysql_ddl('restored_sample177'))
+	
+	#tbl = obj.createload_mysql_from_csv('sample7.csv','restored_sample')
+
+"""	
+	csvfilename = 'a.csv'
+	ddl = obj.guess_postgres_ddl(csvfilename.replace('.','_'))
+	print(ddl)
+	print('/* Tablename used : ' + obj.lastcall_tablename + ' */ \n')
+	print('/* Postgres DDL - END   ----- ----- ----- ----- */ \n')
+	
+	print(obj.dbthings.postgres_db.does_table_exist('newtbl'))
+
+	# add any specific known date formats
+	#obj.dbthings.postgres_date_formats.append('Mon DD,YY')
+	if csvfilename != '':
+		obj.loadcsvfile(csvfilename)
+
+	print('/* MySQL DDL - BEGIN ----- schemawiz().guess_mysql_ddl() ----- */ \n')
+	print(obj.guess_mysql_ddl('sample_csv'))
+	print('/* MySQL DDL - END   ----- ----- ----- ----- */ \n')
+
+
+	print('/* Postgres DDL - BEGIN ----- schemawiz().guess_postgres_ddl() ----- */ \n')
+	ddl = obj.guess_postgres_ddl(csvfilename.replace('.','_'))
+	print('/* Tablename used : ' + obj.lastcall_tablename + ' */ \n')
+	print(ddl)
+	print('/* Postgres DDL - END   ----- ----- ----- ----- */ \n')
+
+
+
+	print('/* BigQuery DDL - BEGIN ----- schemawiz().guess_BigQuery_ddl() ----- */ \n')
+	print(obj.guess_BigQuery_ddl('watchful-lotus-364517','dave'))
+	print('\n/* BigQuery DDL - END   ----- ----- ----- ----- */ \n')
+
+
+	
+	print('/* BigQuery External DDL - BEGIN ----- schemawiz().guess_BigQueryExternal_ddl() ----- */ \n')
+	print(obj.guess_BigQueryExternal_ddl('watchful-lotus-364517','dave'))
+	print('\n/* BigQuery External DDL - END   ----- ----- ----- ----- */ \n')
+
+	print('/* BigQuery DDL - BEGIN ----- schemawiz().guess_BigQuery_ddl() ----- */ \n')
+	print(obj.guess_BigQuery_ddl('watchful-lotus-364517','dave'))
+	print('\n/* BigQuery DDL - END   ----- ----- ----- ----- */ \n')
+
+
+	"""
+
 class database_type:
-    Postgres = 1
-    MySQL = 2
-    BigQuery = 3
+	Postgres = 1
+	MySQL = 2
+	sqlite = 3
+	BigQuery = 4
+
 
 class dbthinger:
 	def __init__(self,date_to_check=''):
+		self.sqlite_db = sqlite_db()
 		self.mysql_db = mysql_db()
 		self.postgres_db = postgres_db()
 		self.postgres_date_formats = ['YYYY/MM/DD','YYYY-MM-DD','YYYY-Mon-DD','MM/DD/YYYY','Mon-DD-YYYY','Mon-DD-YY','Month DD,YY','Month DD,YYYY','DD-Mon-YYYY','YY-Mon-DD','YYYYMMDD','YYMMDD','YYYY-DD-MM','Mon dd/YY']
@@ -30,21 +85,26 @@ class dbthinger:
 	def ask_for_database_details(self,thisDatabaseType):
 		configfilename = '.schemawiz_config' + str(thisDatabaseType)
 
-		DB_HOST = input('DB_HOST (localhost): ') or 'localhost'
+		if thisDatabaseType == database_type.Postgres:
+			DB_NAME = input('DB_NAME (postgres): ') or 'postgres'
+		elif thisDatabaseType == database_type.MySQL:
+			DB_NAME = input('DB_NAME : ') or 'atlas'
+		elif thisDatabaseType == database_type.sqlite:
+			DB_NAME = input('DB_NAME : (local_sqlite_db)') or 'local_sqlite_db'
+
+		DB_HOST = ''
+		if (thisDatabaseType == database_type.Postgres) or (thisDatabaseType == database_type.MySQL):
+			DB_HOST = input('DB_HOST (localhost): ') or 'localhost'
+
 		DB_PORT = ''
 		if thisDatabaseType == database_type.Postgres:
 			DB_PORT = input('DB_PORT (1532): ') or '1532'
 		elif thisDatabaseType == database_type.MySQL:
 			DB_PORT = input('DB_PORT (3306): ') or '3306'
 
-		if thisDatabaseType == database_type.Postgres:
-			DB_NAME = input('DB_NAME (postgres): ') or 'postgres'
-		elif thisDatabaseType == database_type.MySQL:
-			DB_NAME = input('DB_NAME : ') or 'atlas'
-
 		if thisDatabaseType == database_type.Postgres:
 			DB_USERNAME = input('DB_USERNAME (postgres): ') or 'postgres'
-		else:
+		elif thisDatabaseType == database_type.MySQL:
 			DB_USERNAME = input('DB_USERNAME: ') or 'dave'
 
 		DB_SCHEMA = ''
@@ -53,13 +113,15 @@ class dbthinger:
 
 		if thisDatabaseType == database_type.Postgres:
 			DB_USERPWD = input('DB_USERPWD: ') or '4165605869'
-		else:
+		elif thisDatabaseType == database_type.MySQL:
 			DB_USERPWD = input('DB_USERPWD: ') or 'dave'
 
 		if thisDatabaseType == database_type.Postgres:
 			self.postgres_db.useConnectionDetails(DB_USERNAME,DB_USERPWD,DB_HOST,DB_PORT,DB_NAME,DB_SCHEMA)
 		elif thisDatabaseType == database_type.MySQL:
 			self.mysql_db.useConnectionDetails(DB_USERNAME,DB_USERPWD,DB_HOST,DB_PORT,DB_NAME)
+		elif thisDatabaseType == database_type.sqlite:
+			self.sqlite_db.useConnectionDetails(DB_NAME)
 
 		ans_save_connection_details = input('Save connection details? (y/n) :') or 'y'
 		if ans_save_connection_details.upper() == 'Y':
@@ -69,27 +131,36 @@ class dbthinger:
 				f.write(garbledave().garbleit(DB_USERNAME + ' - ' + DB_USERPWD + ' - ' + DB_HOST + ' - ' + DB_PORT + ' - ' + DB_NAME + ' - ' + DB_SCHEMA))
 			elif thisDatabaseType == database_type.MySQL:
 				f.write(garbledave().garbleit(DB_USERNAME + ' - ' + DB_USERPWD + ' - ' + DB_HOST + ' - ' + DB_PORT + ' - ' + DB_NAME))
+			elif thisDatabaseType == database_type.sqlite:
+				f.write(garbledave().garbleit(DB_NAME + ' - '))
 
 			f.close()
 
 	def connect_local_db(self,thisDatabaseType):
 		configfilename = '.schemawiz_config' + str(thisDatabaseType)
-		if ((thisDatabaseType == database_type.Postgres) and (not self.postgres_db.dbconn))	or ((thisDatabaseType == database_type.MySQL) and (not self.mysql_db.dbconn)):
+		if (((thisDatabaseType == database_type.sqlite) and (not self.sqlite_db.dbconn)) or (thisDatabaseType == database_type.Postgres) and (not self.postgres_db.dbconn))	or ((thisDatabaseType == database_type.MySQL) and (not self.mysql_db.dbconn)):
 			try:
 				f = open(configfilename,'r')
 				config_line = garbledave().ungarbleit(f.readline())
-				f.close()
 				dbsettings = config_line.split(' - ')
-				DB_USERNAME = dbsettings[0]
-				DB_USERPWD = dbsettings[1]
-				DB_HOST = dbsettings[2]
-				DB_PORT = dbsettings[3]
-				DB_NAME = dbsettings[4]
-				if thisDatabaseType == database_type.Postgres:
-					DB_SCHEMA = dbsettings[5]
-					self.postgres_db.useConnectionDetails(DB_USERNAME,DB_USERPWD,DB_HOST,DB_PORT,DB_NAME,DB_SCHEMA)
-				elif thisDatabaseType == database_type.MySQL:
-					self.mysql_db.useConnectionDetails(DB_USERNAME,DB_USERPWD,DB_HOST,DB_PORT,DB_NAME)
+				f.close()
+
+				if thisDatabaseType == database_type.sqlite:
+					DB_NAME = dbsettings[0]
+					self.sqlite_db.useConnectionDetails(DB_NAME)
+
+				else:
+					DB_USERNAME = dbsettings[0]
+					DB_USERPWD = dbsettings[1]
+					DB_HOST = dbsettings[2]
+					DB_PORT = dbsettings[3]
+					DB_NAME = dbsettings[4]
+
+					if thisDatabaseType == database_type.Postgres:
+						DB_SCHEMA = dbsettings[5]
+						self.postgres_db.useConnectionDetails(DB_USERNAME,DB_USERPWD,DB_HOST,DB_PORT,DB_NAME,DB_SCHEMA)
+					elif thisDatabaseType == database_type.MySQL:
+						self.mysql_db.useConnectionDetails(DB_USERNAME,DB_USERPWD,DB_HOST,DB_PORT,DB_NAME)
 
 			except Exception as e:
 				self.ask_for_database_details(thisDatabaseType)
@@ -121,6 +192,15 @@ class dbthinger:
 			except:
 					return False
 
+	def is_a_float(self,prm):
+			try:
+				if float(prm) == float(prm):
+					return True
+				else:
+					return False
+			except:
+					return False
+
 	def match_timestamp_type(self,timestamp_string,thisdatabase_type):	
 		if thisdatabase_type == database_type.Postgres:
 			for i in range(0,len(self.postgres_timestamp_formats)):
@@ -210,6 +290,9 @@ class dbthinger:
 		else:
 			return -1
 
+	def match_float_type(self,floatvalue):
+		return self.is_a_float(floatvalue)
+
 	def match_integer_type(self,intvalue):
 		return self.is_an_int(intvalue)
 
@@ -267,6 +350,8 @@ class schemawiz:
 		self.column_names = []
 		self.column_datatypes = []
 		self.BigQuery_datatypes = []
+		self.mysql_datatypes = []
+		self.sqlite_datatypes = []
 		self.column_sample = []
 		self.column_dateformats = []
 		self.analyzed	 = False
@@ -286,9 +371,58 @@ class schemawiz:
 	def newmysqlconnection(self):
 		os.remove('.schemawiz_config' + str(database_type.MySQL))
 
+	def newsqliteconnection(self):
+		os.remove('.schemawiz_config' + str(database_type.sqlite))
+
 	def newconnections(self):
 		self.newpostgresconnection()
 		self.newmysqlconnection()
+		self.newsqliteconnection()
+
+	def justload_sqlite_from_csv(self,csvfilename,tablename,withtruncate=False):
+		return_value = ''
+		self.loadcsvfile(csvfilename)
+		delimiter = self.lastcall_delimiter()
+
+		if self.dbthings.sqlite_db.does_table_exist(tablename):
+			self.dbthings.sqlite_db.load_csv_to_table(csvfilename,tablename,withtruncate,delimiter)
+			rowcount = self.dbthings.sqlite_db.queryone('SELECT COUNT(*) FROM ' + tablename)
+			print('Loaded table ' + tablename + ' with ' + str(rowcount) + ' rows.')
+			return_value = tablename
+		else:
+			print('Table ' + tablename + ' does not exist.  Cannot load. \n alternatively try: createload_postgres_from_csv(csvfilename)')
+
+		return return_value
+
+	def createload_sqlite_from_csv(self,csvfilename,sztablename='',ddlfilename=''):
+		return_value = ''
+
+		self.loadcsvfile(csvfilename)
+		ddl = self.guess_sqlite_ddl(sztablename)
+		delimiter = self.lastcall_delimiter()
+		if sztablename == '':
+			tablename = self.lastcall_tablename
+		else:
+			tablename = sztablename
+
+		if not self.dbthings.sqlite_db.does_table_exist(tablename):
+			if ddlfilename != '':
+				f = open(ddlfilename,'w')
+				f.write(ddl)
+				f.close()
+
+			print(ddl)
+			self.dbthings.sqlite_db.execute(ddl)
+
+			self.dbthings.sqlite_db.load_csv_to_table(csvfilename,tablename,True,delimiter)
+
+			rowcount = self.dbthings.sqlite_db.queryone('SELECT COUNT(*) FROM ' + tablename)
+			print('Created/Loaded table ' + tablename + ' with ' + str(rowcount) + ' rows.')
+			return_value = tablename
+		else:
+			print('Table ' + tablename + ' already exists.  Stopping Load.')
+
+		return return_value
 
 	def justload_postgres_from_csv(self,csvfilename,tablename,withtruncate=False):
 		return_value = ''
@@ -367,7 +501,6 @@ class schemawiz:
 				f.close()
 			print(ddl)
 			self.dbthings.mysql_db.execute(ddl)
-
 			self.dbthings.mysql_db.load_csv_to_table(csvfilename,tablename,True,delimiter)
 
 			rowcount = self.dbthings.mysql_db.queryone('SELECT COUNT(*) FROM ' + tablename)
@@ -448,6 +581,7 @@ class schemawiz:
 				#self.logger('line size is ' + str(self.datalinesize) + ' ytes')
 				#self.logger('file size is ' + str(file_stats.st_size) + ' bytes')
 
+
 				self.get_column_names()
 				self.get_column_types(thisdatabase_type)
 				self.analyzed = True
@@ -521,10 +655,12 @@ class schemawiz:
 		deccount = self.count_decimals(chardict)
 
 		lookslike = ''
-
-		timestamp_nbr = self.dbthings.match_timestamp_type(data,thisdatabase_type)
-		date_format_nbr = self.dbthings.match_date_type(data,thisdatabase_type)
 		dtformat = ''
+		timestamp_nbr = -1
+		date_format_nbr = -1
+		if thisdatabase_type != database_type.sqlite:
+			timestamp_nbr = self.dbthings.match_timestamp_type(data,thisdatabase_type)
+			date_format_nbr = self.dbthings.match_date_type(data,thisdatabase_type)
 
 
 		if timestamp_nbr != -1:
@@ -543,7 +679,11 @@ class schemawiz:
 
 		elif alphacount == 0 and deccount == 1:
 			# 123.123232222
-			lookslike = 'numeric'
+			if self.dbthings.match_float_type(data):
+				lookslike = 'numeric'
+			else:
+				lookslike = 'text'
+
 
 		elif self.dbthings.match_integer_type(data) :
 			# 123
@@ -556,7 +696,6 @@ class schemawiz:
 
 	def get_column_types(self,thisdatabase_type):
 
-
 		found_datatypes = {}
 		found_datavalues = {}
 		found_datefomat = {}
@@ -585,14 +724,14 @@ class schemawiz:
 							found_datatypes[self.column_names[j]] == 'text'
 
 		for k in range(0,len(self.column_names)):
-			if thisdatabase_type == database_type.Postgres:
-				self.column_datatypes.append(found_datatypes[self.column_names[k]])
-			else:
-				self.column_datatypes.append(self.translate_dt(database_type.MySQL,found_datatypes[self.column_names[k]]))
-
 			self.column_sample.append(found_datavalues[self.column_names[k]].replace('"',''))
 			self.column_dateformats.append(found_datefomat[self.column_names[k]])
+
+			self.column_datatypes.append(found_datatypes[self.column_names[k]])
+			self.mysql_datatypes.append(self.translate_dt(database_type.MySQL,found_datatypes[self.column_names[k]]))
+
 			self.BigQuery_datatypes.append(self.translate_dt(database_type.BigQuery,found_datatypes[self.column_names[k]]))
+			self.sqlite_datatypes.append(self.translate_dt(database_type.sqlite,found_datatypes[self.column_names[k]]))
 
 			if not self.IsaDateField and self.translate_dt(database_type.BigQuery,found_datatypes[self.column_names[k]]) == 'DATE': 		
 				self.IsaDateField = True
@@ -628,6 +767,14 @@ class schemawiz:
 				return 'float'
 			else:
 				return postgres_datatype
+		elif targettype == database_type.sqlite:
+			if postgres_datatype.lower().strip() == 'integer':
+				return 'integer'
+			elif postgres_datatype.lower().strip() == 'numeric':
+				return 'real'
+			else:
+				return 'text'
+
 		else:
 			return postgres_datatype
 
@@ -668,21 +815,8 @@ class schemawiz:
 				delimiter_guess = '\t'
 			elif first_row.find(',') > -1:
 				delimiter_guess = ','
-			elif first_row.find('~') > -1:
-				delimiter_guess = '~'
 			else:
-				hdrs = first_row 
-				chars_in_hdr = {}
-				chars_in_hdr = self.count_chars(hdrs,"'"+ 'abcdefghijklmnopqrstuvwxyz1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ"')
-
-				currentchar = 'no idea'
-				currentvalue = 0
-				for ch in chars_in_hdr:
-					if chars_in_hdr[ch] > currentvalue:
-						currentchar = ch
-						currentvalue = chars_in_hdr[ch]
-
-				delimiter_guess = currentchar
+				delimiter_guess = '~'
 
 		return delimiter_guess
 
@@ -717,8 +851,8 @@ class schemawiz:
 		for i in range(0,len(self.column_names)):
 			sql += '\t' + self.column_names[i] + ' ' + self.BigQuery_datatypes[i] + ' \t\t/* eg. ' + self.column_sample[i] + ' */ ' 
 			
-			if self.column_datatypes[i].strip().lower() == 'date' or self.column_datatypes[i].strip().lower() == 'timestamp':
-				sql += "OPTIONS (description='" + self.column_datatypes[i].strip().lower() + " format in csv [" + self.column_dateformats[i] + "]')"
+			if self.BigQuery_datatypes[i].strip().lower() == 'date' or self.BigQuery_datatypes[i].strip().lower() == 'timestamp':
+				sql += "OPTIONS (description='" + self.BigQuery_datatypes[i].strip().lower() + " format in csv [" + self.column_dateformats[i] + "]')"
 
 			sql += ',\n'
 
@@ -762,8 +896,8 @@ class schemawiz:
 		for i in range(0,len(self.column_names)):
 			sql += '\t' + self.column_names[i] + ' ' + self.BigQuery_datatypes[i] + ' \t\t/* eg. ' + self.column_sample[i] + ' */ ' 
 			
-			if self.column_datatypes[i].strip().lower() == 'date' or self.column_datatypes[i].strip().lower() == 'timestamp':
-				sql += "OPTIONS (description='" + self.column_datatypes[i].strip().lower() + " format in csv [" + self.column_dateformats[i] + "]')"
+			if self.BigQuery_datatypes[i].strip().lower() == 'date' or self.BigQuery_datatypes[i].strip().lower() == 'timestamp':
+				sql += "OPTIONS (description='" + self.BigQuery_datatypes[i].strip().lower() + " format in csv [" + self.column_dateformats[i] + "]')"
 
 			sql += ',\n'
 
@@ -789,6 +923,28 @@ class schemawiz:
     
 		return sql
 
+	def guess_sqlite_ddl(self,usetablename=''):
+		self.dbthings.connect_local_db(database_type.sqlite)
+		if not self.analyzed:
+			self.analyze_csvfile(database_type.sqlite)
+		if usetablename == '':
+			tablename = self.gettablename()
+		else:
+			tablename = usetablename
+
+		self.lastcall_tablename = tablename
+
+		fldcommentsql = '' 
+		sql = '/* DROP TABLE IF EXISTS ' + tablename + '; */\n'
+
+		sql += 'CREATE TABLE IF NOT EXISTS ' + tablename + '(\n'
+		for i in range(0,len(self.column_names)):
+			sql += '\t' + self.column_names[i] + ' ' + self.sqlite_datatypes[i] + ' \t\t/* eg. ' + self.column_sample[i] + ' */ ,\n'
+
+		sql = sql[:-2] + '\n);\n\n'
+
+		return sql
+
 	def guess_postgres_ddl(self,usetablename=''):
 		
 		self.dbthings.connect_local_db(database_type.Postgres)
@@ -829,13 +985,13 @@ class schemawiz:
 			tablename = usetablename
 		self.lastcall_tablename = tablename
 
-		sql = 'DROP TABLE IF EXISTS ' + tablename + ';\n'
+		sql = '/* DROP TABLE IF EXISTS ' + tablename + '; */\n'
 
-		sql += 'CREATE TABLE IF NOT EXISTS ' + tablename + '(\n'
+		sql += ' CREATE TABLE IF NOT EXISTS ' + tablename + '(\n'
 		for i in range(0,len(self.column_names)):
-			sql += '\t' + self.column_names[i] + ' ' + str(self.column_datatypes[i]) + ' \t\t/* eg. ' + self.column_sample[i] + ' */ '
-			if str(self.column_datatypes[i]).strip().lower() == 'date' or str(self.column_datatypes[i]).strip().lower() == 'timestamp':
-				sql += 'COMMENT "' + str(self.column_datatypes[i]).strip().lower() + ' format in csv [' + self.column_dateformats[i] + ']" '
+			sql += '\t' + self.column_names[i] + ' ' + str(self.mysql_datatypes[i]) + ' \t\t/* eg. ' + self.column_sample[i] + ' */ '
+			if str(self.mysql_datatypes[i]).strip().lower() == 'date' or str(self.mysql_datatypes[i]).strip().lower() == 'timestamp':
+				sql += 'COMMENT "' + str(self.mysql_datatypes[i]).strip().lower() + ' format in csv [' + self.column_dateformats[i] + ']" '
 			sql += ' ,\n'
 
 		sql = sql[:-2] + '\n) \n'
@@ -846,60 +1002,4 @@ class schemawiz:
 
 
 if __name__ == '__main__':
-	csvfilename ='station_years.tsv' #input('csvfile to read? ')
-
-	obj = schemawiz()
-	#obj.force_delimiter = '\t'
-	#obj.loadcsvfile(csvfilename)
-	#obj.createload_postgres_from_csv(csvfilename,'canweather.station_years','z.canweather.station_years.ddl')
-	
-	#print("delimiter= '" + obj.delimiter + "' ")
-		
-	#ddl = obj.guess_postgres_ddl('canweather.station_events')
-	
-"""
-	print('/* Postgres DDL - BEGIN ----- schemawiz().guess_postgres_ddl() ----- */ \n')
-	ddl = obj.guess_postgres_ddl(csvfilename.replace('.','_'))
-	print('/* Tablename used : ' + obj.lastcall_tablename + ' */ \n')
-	print(ddl)
-	print('/* Postgres DDL - END   ----- ----- ----- ----- */ \n')
-	
-	
-	print(obj.dbthings.postgres_db.does_table_exist('newtbl'))
-
-	# add any specific known date formats
-	#obj.dbthings.postgres_date_formats.append('Mon DD,YY')
-	if csvfilename != '':
-		obj.loadcsvfile(csvfilename)
-
-
-
-	print('/* MySQL DDL - BEGIN ----- schemawiz().guess_mysql_ddl() ----- */ \n')
-	print(obj.guess_mysql_ddl('sample_csv'))
-	print('/* MySQL DDL - END   ----- ----- ----- ----- */ \n')
-
-
-	print('/* Postgres DDL - BEGIN ----- schemawiz().guess_postgres_ddl() ----- */ \n')
-	ddl = obj.guess_postgres_ddl(csvfilename.replace('.','_'))
-	print('/* Tablename used : ' + obj.lastcall_tablename + ' */ \n')
-	print(ddl)
-	print('/* Postgres DDL - END   ----- ----- ----- ----- */ \n')
-
-
-
-	print('/* BigQuery DDL - BEGIN ----- schemawiz().guess_BigQuery_ddl() ----- */ \n')
-	print(obj.guess_BigQuery_ddl('watchful-lotus-364517','dave'))
-	print('\n/* BigQuery DDL - END   ----- ----- ----- ----- */ \n')
-
-
-	
-	print('/* BigQuery External DDL - BEGIN ----- schemawiz().guess_BigQueryExternal_ddl() ----- */ \n')
-	print(obj.guess_BigQueryExternal_ddl('watchful-lotus-364517','dave'))
-	print('\n/* BigQuery External DDL - END   ----- ----- ----- ----- */ \n')
-
-	print('/* BigQuery DDL - BEGIN ----- schemawiz().guess_BigQuery_ddl() ----- */ \n')
-	print(obj.guess_BigQuery_ddl('watchful-lotus-364517','dave'))
-	print('\n/* BigQuery DDL - END   ----- ----- ----- ----- */ \n')
-
-
-	"""
\ No newline at end of file
+	main()
